/*
 * Copyright (C) 2001 LAAS/CNRS 
 *
 * Anthony Mallet - Wed Jul  4 2001
 *
 * $Log: libp3d.h,v $
 * Revision 1.6  2007/09/10 10:55:22  sjoyeux
 * return the found path instead than the first arc of the path
 *
 *   Instead of returning only the first arc, the search functions now assume
 *   that bestArc is an array if P3DArc structures of at least depth elements.
 *   This array is filled with the arcs composing the path, starting by the
 *   arc nearest to the goal.
 *
 *   The new output argument solutionDepth contains the number of arcs actually
 *   found in the solution path.
 *
 * Revision 1.5  2007/06/28 17:03:58  tpeynot
 *
 * Add fonctions used for p3d monitoring (import from the old genom version)
 *
 * Revision 1.4  2007/06/28 11:46:12  tpeynot
 *
 * Add a sampling rate as a parameter of the main function p3dDifmap (the one that actually builds a difmap from a p3dmap)
 * This parameter is an int giving the sampling rate with respect to the p3dmap cells (i.e. sampling rate in the dtm poster, which is the same)
 *
 * Revision 1.3  2007/06/18 17:37:19  tpeynot
 * Add structures and functions to manipulate difficulty maps (difmap)
 *
 * Revision 1.2  2007/01/08 19:55:17  sjoyeux
 * * sync with the DEM tag (was something else before)
 * * update to OpenRobot's T3Ds
 *
 * Revision 1.4.4.15  2006/04/28 16:17:27  tpeynot
 * Creation of function probaDifmap to compute the cells partial probabilities of a whole difmap + Adaptation of difmap_2_imaf to be able to display these probas
 *
 * Revision 1.4.4.14  2006/04/27 16:30:36  tpeynot
 * proto of fuseDifmap added
 *
 * Revision 1.4.4.13  2006/04/18 12:39:08  tpeynot
 * Function difmapAllocateDifficulty added: defines nbLines and nbCols in the DIFMAP structure and allocates the difficulty array accordingly
 *
 * Revision 1.4.4.12  2006/02/28 12:51:45  tpeynot
 * function read_difmap added
 *
 * Revision 1.4.4.11  2006/02/01 13:12:50  tpeynot
 * protos of functions filterDifmap and copy_difmap added
 *
 * Revision 1.4.4.10  2005/10/28 12:22:55  tpeynot
 * functions for P3dRobotConfig object added + config added in p3dPlaceRobotOnMap prototype
 *
 * Revision 1.4.4.9  2005/07/13 15:59:57  tpeynot
 * prototypes of functions used for difmaps added
 *
 * Revision 1.4.4.8  2005/05/02 17:29:17  tpeynot
 * some Functions now return ErrorCode
 * and proto of funtion p3dDisplayP3DArcConfig added
 *
 * Revision 1.4.4.7  2005/04/21 12:39:29  tpeynot
 * Add protos of gdhe connection functions
 * Add display param to p3dPlaceRobotDala proto
 *
 * Revision 1.4.4.6  2005/04/20 16:50:51  tpeynot
 * Add destroy_p3dmap fct proto
 * and add p3dPlaceRobotDala fct proto to be able to use it outside of p3dCalife, separately
 *
 * Revision 1.4.4.5  2005/04/13 23:20:03  tpeynot
 * Add elements in protos for body collision test
 * and add display functions protos
 *
 * Revision 1.4.4.4  2005/03/29 09:38:15  tpeynot
 * Display Functions prototypes added
 *
 * Revision 1.4.4.3  2005/03/21 23:07:39  tpeynot
 * Add prototypes of functions create_p3dmap and write_p3dmap
 *
 * Revision 1.4.4.2  2005/03/18 23:47:44  tpeynot
 * Declaration of struct P3D_MAP in p3d.h
 * and take it into account in p3dPlan prototype in libp3d.h
 *
 * Revision 1.4.4.1  2005/03/18 14:09:50  tpeynot
 * Tag DEM (p3d version linked to DEM instead of Lane)
 *
 * Revision 1.4  2002/10/09 15:45:04  mallet
 * p3dPlan retourne le cout estime, normalise entre 1 et +infini
 *
 * Revision 1.3  2001/07/14 11:38:48  mallet
 * premiere version ok
 *
 * Revision 1.2  2001/07/12 17:38:44  mallet
 * sync
 *
 * Revision 1.1  2001/07/11 18:06:14  mallet
 * headers pour p3d
 *
 */
#ifndef LIBP3D_H
#define LIBP3D_H

#include "p3d.h"

#include "difmap.h"

#include <stdbool.h>

#ifdef __cplusplus
extern "C" {
#endif

/*---- Exported Functions --------------------------*/

P3DPlanner *	p3dCreatePlanner(void);
void		p3dDestroyPlanner(P3DPlanner *planner);

int		p3dConfigureGeom(P3DPlanner *planner, double len[3],
			double rad[2], double arm[2],
			double bodyLength, double bodyWidth, 
			double zBody, double groundClear,
			double rollLimits[3], double pitchLimits[3],
			double alphaLimits[3], double betaLimits[3],
			double unkMax);
int		p3dConfigureRobot(P3DPlanner *planner, double scale,
			double rollStep, double vMin, double vMax, 
			double wMax, double accel, double decel);

int		p3dConfigureRobotFromFile(P3DPlanner *planner, const char * name);
int		p3dRobotSaveToFile(P3DPlanner *planner, const char *name);

int		p3dConfigureArcs(P3DPlanner *planner, int nArcs,
			int nNodes, int depth, double rMin, double goalTol, double dMin); 

int		p3dPlan(P3DPlanner *planner, P3D_MAP *map, 
			T3D *mainToOrigin, T3D *goalToOrigin,
			void (*callback)(P3DNode *node, void *data),
			void *data, enum P3DWay way, P3DPlanSolution * sol);

int		p3dDumpP3dArcs(P3DArc *arcs, T3D* mainToOrigin, int n, P3DArcDump *bestArc);

/*----- p3dPlanSolution --------------*/

P3DPlanSolution * create_P3DPlanSolution(int depth, int nNodes);

void			  destroy_P3DPlanSolution(P3DPlanSolution *sol);

void			  reset_P3DPlanSolution(P3DPlanSolution *sol);

/*----- p3dMap -----------------------*/
P3D_MAP*	create_p3dmap();
void            destroy_p3dmap(P3D_MAP*  p3dMap);

int				read_p3dmap(P3D_MAP* map, const char* filename);
int				write_p3dmap(const P3D_MAP*  map, const char* filename);

int				p3dmap_to_gdhe(const P3D_MAP* map, const char* filename);
			     

/*----- p3dRobotConfig -----------------------*/
P3DRobotConfig* p3dCreateRobotConfig();
void            p3dDestroyRobotConfig(P3DRobotConfig *config);
void            p3dInfoP3DRobotConfig(P3DRobotConfig*  config);


/*----------------------------*/
/* proto originally in p3dInt.h, but function needed outside now */
int p3dPlaceRobotOnMap(P3DRobotConfig*  config,
			     P3DPlanner* p3dplanner,
			     double robot_x, double robot_y, 
			     double robot_theta,
			     P3D_MAP*   p3dMap,
			     int    display);

int p3dPlaceRobotOnWholeMap(P3DPlanner* p3dplanner,
				  double thetaDeg,
				  P3D_MAP*   map,
				  double    x_min,
				  double    y_min,
				  double    x_max,
				  double    y_max );

/*-------------------- Display Functions (GDHE) -------------------*/

int p3dDispCreateFirstWayPoint(float diameter, float height);

int p3dDispCreateWayPoints(int wayptNb, float diameter, float height);

int p3dDispSetPosWaypoint(int i, float x, float y);

int p3dDispSetPosArc(double theta, double x, double y, double z);

int p3dDispDrawSpeedRefArc(double vref, double wref,
			   int numNodes, double length);

int p3dDispSetPosDala(double theta, double pitch, double roll,
		      double x, double y, double z );

int p3dDispCreateMiniDalas(int    robotsNb,
				 float  scale );

int p3dDisplayOnePredictedConfig(T3D*       nodePos,
				 double     configPitch,
				 double     configRoll,
				 double     configZ,
				 T3D*       robotPos,
				 int        nodeNb,
				 float      scale );

int p3dDisplayP3DArcConfig (P3DArc*  currentArc,
				  T3D*  robotToOrigin,
				  int   firstNodeNb,
				  int   lastNodeNb,
				  int   interval );

/* GDHE Connection */
int p3d_gdheGetConnection (char*	serverName);

int p3d_gdheDisconnect ( );

int p3d_gdheEvalExpression (char*	expression);


/* Functions for p3d Monitor */
double p3dShapeDistrib (double    dist,     
			double    maxDist,  
			double    obs,      
			double    maxAngle, 
			double    sigmaAngle);


/*----- Functions for difMaps ---------------*/

DIF_MAP*  create_difmap ();

int difmapAllocateDifficulty (DIF_MAP*  difmap,
				    int       nbLines,
				    int       nbCols);

void destroy_difmap (DIF_MAP*  difMap);

short write_difmap(DIF_MAP*  map,
		   char      fileName[1024]);

DIF_MAP* read_difmap(char      fileName[1024]);

DIF_MAP*  copy_difmap (DIF_MAP*  difmap);

DIF_MAP*  filterDifmap (DIF_MAP*  difmap,
			int      numPixSigma );


int p3dDifMap(DIF_MAP  *difMap,
	      P3D_MAP  *p3dMap,
	      int       delta,
	      P3DRobot *robot, 
	      P3DRobotGeometry *geom);

int  fuseDifmap (DIF_MAP*  globalDifmap,
		       DIF_MAP*  localDifmap); 

int  probaDifmap (DIF_MAP*  difmap,
			double    avRough,
			double    sigmaFlat,
			double    sigmaRough,
			double    sigmaObstacle);

DIF_MAP*  patchDifmap (DIF_MAP*  difmap,
		       int       discrRate);

int getDifmapProbas(DIF_MAP*   difmap,  
			  double     x,
			  double     y,  
			  double     probas[DIFMAP_NB_CLASSES]);

double getDifmapDifficulty(DIF_MAP*   difmap,
			   double     x,
			   double     y );


/*-------- Functions for ACO -----------------*/

#if defined(TROOPER)
bool
trajectory_is_safe(P3DPlanner *planner, P3D_MAP *map, 
			double robot_x, double robot_y, double robot_theta, double alpha);

int
p3dAco(P3DPlanner *planner, P3D_MAP *map, 
		T3D *mainToOrigin, P3DPlanSolution * sol,
		double alpha, double v, double max_angle, double min_angle);

int
p3dEvo(P3DPlanner *planner, P3D_MAP *map,
	T3D *mainToOrigin, double alpha, double v,
	P3DPlanSolution * sol);
#endif

#ifdef __cplusplus
}
#endif

#endif /* LIBP3D_H */
